from __future__ import annotations
from yamloom.actions.utils import validate_choice

from typing import TYPE_CHECKING

from ..._yamloom import Step
from ..._yamloom import action
from ..types import (
    Oboollike,
    Oboolstr,
    Ointlike,
    Ostr,
    Ostrlike,
    StringLike,
    BoolLike,
)

if TYPE_CHECKING:
    from collections.abc import Mapping

__all__ = ['setup_node', 'setup_pnpm']


def setup_node(
    *,
    name: Ostrlike = None,
    version: str = 'v6',
    node_version: Ostrlike = None,
    node_version_file: Ostrlike = None,
    check_latest: Oboollike = None,
    architecture: Ostrlike = None,
    token: Ostrlike = None,
    cache: Ostrlike = None,
    package_manager_cache: Oboollike = None,
    cache_dependency_path: Ostrlike = None,
    registry_url: Ostrlike = None,
    scope: Ostrlike = None,
    mirror: Ostrlike = None,
    mirror_token: Ostrlike = None,
    args: Ostrlike = None,
    entrypoint: Ostrlike = None,
    condition: Oboolstr = None,
    working_directory: Ostrlike = None,
    shell: Ostr = None,
    id: Ostr = None,  # noqa: A002
    env: Mapping[str, StringLike] | None = None,
    continue_on_error: Oboollike = None,
    timeout_minutes: Ointlike = None,
) -> Step:
    options: dict[str, object] = {
        'node-version': node_version,
        'node-version-file': node_version_file,
        'check-latest': check_latest,
        'architecture': architecture,
        'token': token,
        'cache': validate_choice('cache', cache, ['npm', 'yarn', 'pnpm']),
        'package-manager-cache': package_manager_cache,
        'cache-dependency-path': cache_dependency_path,
        'registry-url': registry_url,
        'scope': scope,
        'mirror': mirror,
        'mirror-token': mirror_token,
    }

    options = {key: value for key, value in options.items() if value is not None}

    if name is None:
        name = 'Setup Node'

    return action(
        name,
        'actions/setup-node',
        ref=version,
        with_opts=options or None,
        args=args,
        entrypoint=entrypoint,
        condition=condition,
        working_directory=working_directory,
        shell=shell,
        id=id,
        env=env,
        continue_on_error=continue_on_error,
        timeout_minutes=timeout_minutes,
    )


def setup_pnpm(
    *,
    name: Ostrlike = None,
    version: str = 'v4',
    pnpm_version: Ostrlike = None,
    dest: Ostrlike = None,
    run_install: StringLike | BoolLike | None = None,
    cache: Oboollike = None,
    cache_dependency_path: Ostrlike | list[StringLike] = None,
    package_json_file: Ostrlike = None,
    standalone: Oboollike = None,
    args: Ostrlike = None,
    entrypoint: Ostrlike = None,
    condition: Oboolstr = None,
    working_directory: Ostrlike = None,
    shell: Ostr = None,
    id: Ostr = None,  # noqa: A002
    env: Mapping[str, StringLike] | None = None,
    continue_on_error: Oboollike = None,
    timeout_minutes: Ointlike = None,
) -> Step:
    options: dict[str, object] = {
        'version': pnpm_version,
        'dest': dest,
        'run_install': run_install,
        'cache': cache,
        'cache_dependency_path': cache_dependency_path,
        'package_json_file': package_json_file,
        'standalone': standalone,
    }

    options = {key: value for key, value in options.items() if value is not None}

    if name is None:
        name = 'Setup pnpm'

    return action(
        name,
        'pnpm/action-setup',
        ref=version,
        with_opts=options or None,
        args=args,
        entrypoint=entrypoint,
        condition=condition,
        working_directory=working_directory,
        shell=shell,
        id=id,
        env=env,
        continue_on_error=continue_on_error,
        timeout_minutes=timeout_minutes,
    )



from typing import ClassVar, Union

BoolLike = Union[bool, BooleanExpression]
NumberLike = Union[float, int, NumberExpression]
StringLike = Union[str, StringExpression]

class BooleanExpression:
    def as_num(self) -> NumberExpression: ...
    def as_str(self) -> StringExpression: ...
    def as_obj(self) -> ObjectExpression: ...
    def __invert__(self) -> BooleanExpression: ...
    def __and__(self, other: BoolLike) -> BooleanExpression: ...
    def __or__(self, other: BoolLike) -> BooleanExpression: ...
    def __eq__(self, other: BoolLike) -> BooleanExpression: ...  # ty:ignore[invalid-method-override]
    def __ne__(self, other: BoolLike) -> BooleanExpression: ...  # ty:ignore[invalid-method-override]
    def if_else(
        self, condition: BoolLike, else_expr: BoolLike
    ) -> BooleanExpression: ...
    def to_json(self) -> ObjectExpression: ...

class NumberExpression:
    def as_bool(self) -> BooleanExpression: ...
    def as_str(self) -> StringExpression: ...
    def as_obj(self) -> ObjectExpression: ...
    def __lt__(self, other: NumberLike) -> BooleanExpression: ...
    def __le__(self, other: NumberLike) -> BooleanExpression: ...
    def __gt__(self, other: NumberLike) -> BooleanExpression: ...
    def __ge__(self, other: NumberLike) -> BooleanExpression: ...
    def __eq__(self, other: NumberLike) -> BooleanExpression: ...  # ty:ignore[invalid-method-override]
    def __ne__(self, other: NumberLike) -> BooleanExpression: ...  # ty:ignore[invalid-method-override]
    def if_else(
        self, condition: BoolLike, else_expr: NumberLike
    ) -> NumberExpression: ...
    def to_json(self) -> ObjectExpression: ...

class StringExpression:
    def as_bool(self) -> BooleanExpression: ...
    def as_num(self) -> NumberExpression: ...
    def as_obj(self) -> ObjectExpression: ...
    def __eq__(self, other: StringLike) -> BooleanExpression: ...  # ty:ignore[invalid-method-override]
    def __ne__(self, other: StringLike) -> BooleanExpression: ...  # ty:ignore[invalid-method-override]
    def contains(self, other: StringLike) -> BooleanExpression: ...
    def startswith(self, other: StringLike) -> BooleanExpression: ...
    def endswith(self, other: StringLike) -> BooleanExpression: ...
    def format(self, args: list[StringLike]) -> StringExpression: ...
    def to_json(self) -> ObjectExpression: ...
    def from_json_to_bool(self) -> BooleanExpression: ...
    def from_json_to_num(self) -> NumberExpression: ...
    def from_json_to_str(self) -> StringExpression: ...
    def from_json_to_array(self) -> ArrayExpression: ...
    def from_json_to_obj(self) -> ObjectExpression: ...
    def hash_files(
        self, others: list[StringLike] | None = None
    ) -> StringExpression: ...
    def if_else(
        self, condition: BoolLike, else_expr: StringLike
    ) -> StringExpression: ...

class ArrayExpression:
    def as_num(self) -> NumberExpression: ...
    def as_obj(self) -> ObjectExpression: ...
    def contains(self, other: ObjectExpression) -> BooleanExpression: ...
    def join(self, separator: StringLike | None = None) -> StringExpression: ...
    def to_json(self) -> ObjectExpression: ...

class ObjectExpression:
    def as_num(self) -> NumberExpression: ...
    def as_str(self) -> StringExpression: ...
    def as_bool(self) -> BooleanExpression: ...
    def as_array(self) -> ArrayExpression: ...
    def to_json(self) -> ObjectExpression: ...
    def from_json_to_bool(self) -> BooleanExpression: ...
    def from_json_to_num(self) -> NumberExpression: ...
    def from_json_to_str(self) -> ObjectExpression: ...
    def from_json_to_array(self) -> ArrayExpression: ...
    def from_json_to_obj(self) -> ObjectExpression: ...
    def __getitem__(self, key: str) -> ObjectExpression: ...
    def __getattr__(self, key: str) -> ObjectExpression: ...
    def get(self, key: str) -> ObjectExpression: ...

class GithubContext:
    @property
    def expr(self) -> ObjectExpression: ...
    @property
    def action(self) -> StringExpression: ...
    @property
    def action_path(self) -> StringExpression: ...
    @property
    def action_ref(self) -> StringExpression: ...
    @property
    def action_repository(self) -> StringExpression: ...
    @property
    def action_status(self) -> StringExpression: ...
    @property
    def actor(self) -> StringExpression: ...
    @property
    def actor_id(self) -> StringExpression: ...
    @property
    def api_url(self) -> StringExpression: ...
    @property
    def base_ref(self) -> StringExpression: ...
    @property
    def env(self) -> StringExpression: ...
    @property
    def event(self) -> ObjectExpression: ...
    @property
    def event_name(self) -> StringExpression: ...
    @property
    def event_path(self) -> StringExpression: ...
    @property
    def graphql_url(self) -> StringExpression: ...
    @property
    def head_ref(self) -> StringExpression: ...
    @property
    def job(self) -> StringExpression: ...
    @property
    def path(self) -> StringExpression: ...
    @property
    def ref(self) -> StringExpression: ...
    @property
    def ref_name(self) -> StringExpression: ...
    @property
    def ref_protected(self) -> BooleanExpression: ...
    @property
    def ref_type(self) -> StringExpression: ...
    @property
    def repository(self) -> StringExpression: ...
    @property
    def reporitory_id(self) -> StringExpression: ...
    @property
    def repositor_owner(self) -> StringExpression: ...
    @property
    def repository_owner_id(self) -> StringExpression: ...
    @property
    def repository_url(self) -> StringExpression: ...
    @property
    def retention_days(self) -> StringExpression: ...
    @property
    def run_id(self) -> StringExpression: ...
    @property
    def run_number(self) -> StringExpression: ...
    @property
    def run_attempt(self) -> StringExpression: ...
    @property
    def secret_source(self) -> StringExpression: ...
    @property
    def server_url(self) -> StringExpression: ...
    @property
    def sha(self) -> StringExpression: ...
    @property
    def token(self) -> StringExpression: ...
    @property
    def triggering_actor(self) -> StringExpression: ...
    @property
    def workflow(self) -> StringExpression: ...
    @property
    def workflow_ref(self) -> StringExpression: ...
    @property
    def workflow_sha(self) -> StringExpression: ...
    @property
    def workspace(self) -> StringExpression: ...

class EnvContext:
    @property
    def expr(self) -> ObjectExpression: ...
    def __getitem__(self, key: str) -> StringExpression: ...
    def __getattr__(self, key: str) -> StringExpression: ...

class VarsContext:
    @property
    def expr(self) -> ObjectExpression: ...
    def __getitem__(self, key: str) -> StringExpression: ...
    def __getattr__(self, key: str) -> StringExpression: ...

class JobContainerContext:
    @property
    def expr(self) -> ObjectExpression: ...
    @property
    def id(self) -> StringExpression: ...
    @property
    def network(self) -> StringExpression: ...

class JobServicesIdContext:
    @property
    def expr(self) -> ObjectExpression: ...
    @property
    def id(self) -> StringExpression: ...
    @property
    def network(self) -> StringExpression: ...
    @property
    def ports(self) -> ObjectExpression: ...

class JobServicesContext:
    @property
    def expr(self) -> ObjectExpression: ...
    def __getitem__(self, key: str) -> JobServicesIdContext: ...
    def __getattr__(self, key: str) -> JobServicesIdContext: ...

class JobContext:
    @property
    def expr(self) -> ObjectExpression: ...
    @property
    def check_run_id(self) -> NumberExpression: ...
    @property
    def container(self) -> JobContainerContext: ...
    @property
    def services(self) -> JobServicesContext: ...
    @property
    def status(self) -> StringExpression: ...

class JobsJobIdOutputsContext:
    @property
    def expr(self) -> ObjectExpression: ...
    def __getitem__(self, key: str) -> StringExpression: ...
    def __getattr__(self, key: str) -> StringExpression: ...

class JobsJobIdContext:
    @property
    def expr(self) -> ObjectExpression: ...
    @property
    def result(self) -> StringExpression: ...
    @property
    def outputs(self) -> JobsJobIdOutputsContext: ...

class JobsContext:
    @property
    def expr(self) -> ObjectExpression: ...
    def __getitem__(self, key: str) -> JobsJobIdContext: ...
    def __getattr__(self, key: str) -> JobsJobIdContext: ...

class StepsStepIdOutputsContext:
    @property
    def expr(self) -> ObjectExpression: ...
    def __getitem__(self, key: str) -> StringExpression: ...
    def __getattr__(self, key: str) -> StringExpression: ...

class StepsStepIdContext:
    @property
    def expr(self) -> ObjectExpression: ...
    @property
    def outputs(self) -> StepsStepIdOutputsContext: ...
    @property
    def conclusion(self) -> StringExpression: ...
    @property
    def outcome(self) -> StringExpression: ...

class StepsContext:
    @property
    def expr(self) -> ObjectExpression: ...
    def __getitem__(self, key: str) -> StepsStepIdContext: ...
    def __getattr__(self, key: str) -> StepsStepIdContext: ...

class RunnerContext:
    @property
    def expr(self) -> ObjectExpression: ...
    @property
    def name(self) -> StringExpression: ...
    @property
    def os(self) -> StringExpression: ...
    @property
    def arch(self) -> StringExpression: ...
    @property
    def temp(self) -> StringExpression: ...
    @property
    def tool_cache(self) -> StringExpression: ...
    @property
    def debug(self) -> StringExpression: ...
    @property
    def environment(self) -> StringExpression: ...

class SecretsContext:
    @property
    def expr(self) -> ObjectExpression: ...
    @property
    def github_token(self) -> StringExpression: ...
    def __getitem__(self, key: str) -> StringExpression: ...
    def __getattr__(self, key: str) -> StringExpression: ...

class StrategyContext:
    @property
    def expr(self) -> ObjectExpression: ...
    @property
    def fail_fast(self) -> BooleanExpression: ...
    @property
    def job_index(self) -> NumberExpression: ...
    @property
    def job_total(self) -> NumberExpression: ...
    @property
    def max_parallel(self) -> NumberExpression: ...

class MatrixContext:
    @property
    def expr(self) -> ObjectExpression: ...
    def __getitem__(self, key: str) -> ObjectExpression: ...
    def __getattr__(self, key: str) -> ObjectExpression: ...

class NeedsJobIdOutputsContext:
    @property
    def expr(self) -> ObjectExpression: ...
    def __getitem__(self, key: str) -> StringExpression: ...
    def __getattr__(self, key: str) -> StringExpression: ...

class NeedsJobIdContext:
    @property
    def expr(self) -> ObjectExpression: ...
    @property
    def outputs(self) -> NeedsJobIdOutputsContext: ...
    @property
    def result(self) -> StringExpression: ...

class NeedsContext:
    @property
    def expr(self) -> ObjectExpression: ...
    def __getitem__(self, key: str) -> NeedsJobIdContext: ...
    def __getattr__(self, key: str) -> NeedsJobIdContext: ...

class InputsContext:
    @property
    def expr(self) -> ObjectExpression: ...
    def __getitem__(self, key: str) -> ObjectExpression: ...
    def __getattr__(self, key: str) -> ObjectExpression: ...

class context:  # noqa: N801
    github: ClassVar[GithubContext]
    env: ClassVar[EnvContext]
    vars: ClassVar[VarsContext]
    job: ClassVar[JobContext]
    jobs: ClassVar[JobsContext]
    steps: ClassVar[StepsContext]
    runner: ClassVar[RunnerContext]
    secrets: ClassVar[SecretsContext]
    strategy: ClassVar[StrategyContext]
    matrix: ClassVar[MatrixContext]
    needs: ClassVar[NeedsContext]
    inputs: ClassVar[InputsContext]

def success() -> BooleanExpression: ...
def always() -> BooleanExpression: ...
def cancelled() -> BooleanExpression: ...
def failure() -> BooleanExpression: ...
def lit_str(s: str) -> StringExpression: ...
def lit_bool(b: bool) -> BooleanExpression: ...
def lit_num(n: float) -> NumberExpression: ...

__all__ = [
    'ArrayExpression',
    'BooleanExpression',
    'NumberExpression',
    'ObjectExpression',
    'StringExpression',
    'always',
    'cancelled',
    'context',
    'failure',
    'lit_bool',
    'lit_num',
    'lit_str',
    'success',
]



from __future__ import annotations
from yamloom.actions.utils import validate_choice

from typing import TYPE_CHECKING

from ..._yamloom import Step
from ..._yamloom import action
from ..types import (
    Oboollike,
    Oboolstr,
    Ointlike,
    Ostr,
    Ostrlike,
    StringLike,
)

if TYPE_CHECKING:
    from collections.abc import Mapping

__all__ = ['setup_java']


def setup_java(
    *,
    name: Ostrlike = None,
    version: str = 'v5',
    java_version: Ostrlike = None,
    java_version_file: Ostrlike = None,
    distribution: Ostrlike = None,
    java_package: Ostrlike = None,
    check_latest: Oboollike = None,
    architecture: Ostrlike = None,
    jdk_file: Ostrlike = None,
    cache: Ostrlike = None,
    cache_dependency_path: Ostrlike = None,
    overwrite_settings: Oboollike = None,
    server_id: Ostrlike = None,
    server_username: Ostrlike = None,
    server_password: Ostrlike = None,
    settings_path: Ostrlike = None,
    gpg_private_key: Ostrlike = None,
    gpg_passphrase: Ostrlike = None,
    mvn_toolchain_id: Ostrlike = None,
    mvn_toolchain_vendor: Ostrlike = None,
    args: Ostrlike = None,
    entrypoint: Ostrlike = None,
    condition: Oboolstr = None,
    working_directory: Ostrlike = None,
    shell: Ostr = None,
    id: Ostr = None,  # noqa: A002
    env: Mapping[str, StringLike] | None = None,
    continue_on_error: Oboollike = None,
    timeout_minutes: Ointlike = None,
) -> Step:
    options: dict[str, object] = {
        'java-version': java_version,
        'java-version-file': java_version_file,
        'distribution': distribution,
        'java-package': validate_choice(
            'java_package', java_package, ['jdk', 'jre', 'jdk+fx', 'jre+fx']
        ),
        'check-latest': check_latest,
        'architecture': validate_choice(
            'architecture', architecture, ['x86', 'x64', 'armv7', 'aarch64', 'ppc64le']
        ),
        'jdkFile': jdk_file,
        'cache': validate_choice('cache', cache, ['maven', 'gradle', 'sbt']),
        'cache-dependency-path': cache_dependency_path,
        'overwrite-settings': overwrite_settings,
        'server-id': server_id,
        'server-username': server_username,
        'server-password': server_password,
        'settings-path': settings_path,
        'gpg-private_key': gpg_private_key,
        'gpg-passphrase': gpg_passphrase,
        'mvn-toolchain-id': mvn_toolchain_id,
        'mvn-toolchain-vendor': mvn_toolchain_vendor,
    }
    options = {key: value for key, value in options.items() if value is not None}

    if name is None:
        name = 'Setup Java'

    return action(
        name,
        'actions/setup-java',
        ref=version,
        with_opts=options or None,
        args=args,
        entrypoint=entrypoint,
        condition=condition,
        working_directory=working_directory,
        shell=shell,
        id=id,
        env=env,
        continue_on_error=continue_on_error,
        timeout_minutes=timeout_minutes,
    )



from __future__ import annotations

from typing import TYPE_CHECKING

from ..._yamloom import Step
from ..._yamloom import action
from ..types import (
    Oboollike,
    Oboolstr,
    Ointlike,
    Ostr,
    Ostrlike,
    StringLike,
)

if TYPE_CHECKING:
    from collections.abc import Mapping

__all__ = ['maturin']


def maturin(
    *,
    name: Ostrlike = None,
    version: str = 'v1',
    token: Ostrlike = None,
    command: Ostrlike = None,
    maturin_version: Ostrlike = None,
    manylinux: Ostrlike = None,
    container: Ostrlike = None,
    docker_options: Ostrlike = None,
    host_home_mount: Ostrlike = None,
    target: Ostrlike = None,
    rust_toolchain: Ostrlike = None,
    rustup_components: list[StringLike] | None = None,
    maturin_working_directory: Ostrlike = None,
    sccache: Oboollike = None,
    before_script_linux: Ostrlike = None,
    args: Ostrlike = None,
    entrypoint: Ostrlike = None,
    condition: Oboolstr = None,
    working_directory: Ostrlike = None,
    shell: Ostr = None,
    id: Ostr = None,  # noqa: A002
    env: Mapping[str, StringLike] | None = None,
    continue_on_error: Oboollike = None,
    timeout_minutes: Ointlike = None,
) -> Step:
    options: dict[str, object] = {
        'command': command,
        'maturin-version': maturin_version,
        'manylinux': manylinux,
        'target': target,
        'container': container,
        'docker-options': docker_options,
        'rust-toolchain': rust_toolchain,
        'rustup-components': ','.join(str(s) for s in rustup_components)
        if rustup_components is not None
        else None,
        'working-directory': maturin_working_directory,
        'sccache': sccache,
        'before-script-linux': before_script_linux,
    }

    options = {key: value for key, value in options.items() if value is not None}

    if name is None:
        name = 'Maturin Action'

    return action(
        name,
        'PyO3/maturin-action',
        ref=version,
        with_opts=options or None,
        args=args,
        entrypoint=entrypoint,
        condition=condition,
        working_directory=working_directory,
        shell=shell,
        id=id,
        env=env,
        continue_on_error=continue_on_error,
        timeout_minutes=timeout_minutes,
    )



from __future__ import annotations
from yamloom.actions.utils import check_string

from typing import TYPE_CHECKING

from ..._yamloom import Step
from ..._yamloom import action
from ..types import (
    Oboollike,
    Oboolstr,
    Ointlike,
    Ostr,
    Ostrlike,
    StringLike,
)

if TYPE_CHECKING:
    from collections.abc import Mapping

__all__ = ['checkout']


def checkout(
    *,
    name: Ostrlike = None,
    version: str = 'v6',
    repository: Ostrlike = None,
    ref: Ostrlike = None,
    token: Ostrlike = None,
    ssh_key: Ostrlike = None,
    ssh_known_hosts: Ostrlike = None,
    ssh_strict: Oboollike = None,
    ssh_user: Ostrlike = None,
    persist_credentials: Oboollike = None,
    path: Ostrlike = None,
    clean: Oboollike = None,
    filter: Ostrlike = None,  # noqa: A002
    sparse_checkout: Ostrlike = None,
    sparse_checkout_cone_mode: Oboollike = None,
    fetch_depth: Ointlike = None,
    fetch_tags: Oboollike = None,
    show_progress: Oboollike = None,
    lfs: Oboollike = None,
    submodules: Oboollike = None,
    get_safe_directory: Oboollike = None,
    github_server_url: Ostrlike = None,
    args: Ostrlike = None,
    entrypoint: Ostrlike = None,
    condition: Oboolstr = None,
    working_directory: Ostrlike = None,
    shell: Ostr = None,
    id: Ostr = None,  # noqa: A002
    env: Mapping[str, StringLike] | None = None,
    continue_on_error: Oboollike = None,
    timeout_minutes: Ointlike = None,
) -> Step:
    options: dict[str, object] = {
        'repository': repository,
        'ref': ref,
        'token': token,
        'ssh-key': ssh_key,
        'ssh-known-hosts': ssh_known_hosts,
        'ssh-strict': ssh_strict,
        'ssh-user': ssh_user,
        'persist-credentials': persist_credentials,
        'path': path,
        'clean': clean,
        'filter': filter,
        'sparse-checkout': sparse_checkout,
        'sparse-checkout-cone-mode': sparse_checkout_cone_mode,
        'fetch-depth': fetch_depth,
        'fetch-tags': fetch_tags,
        'show-progress': show_progress,
        'lfs': lfs,
        'submodules': submodules,
        'get-safe-directory': get_safe_directory,
        'github-server-url': github_server_url,
    }
    options = {key: value for key, value in options.items() if value is not None}

    if name is None:
        repository_str = check_string(options.get('repository'))
        if repository_str:
            name = f"Checkout '{repository_str}'"
        else:
            name = 'Checkout Repository'

    return action(
        name,
        'actions/checkout',
        ref=version,
        with_opts=options or None,
        args=args,
        entrypoint=entrypoint,
        condition=condition,
        working_directory=working_directory,
        shell=shell,
        id=id,
        env=env,
        continue_on_error=continue_on_error,
        timeout_minutes=timeout_minutes,
    )



from __future__ import annotations
from yamloom.actions.utils import validate_choice, check_string

from typing import TYPE_CHECKING

from ..._yamloom import Step
from ..._yamloom import action
from ..types import (
    Oboollike,
    Oboolstr,
    Ointlike,
    Ostr,
    Ostrlike,
    StringLike,
)

if TYPE_CHECKING:
    from collections.abc import Mapping

__all__ = ['setup_mpi']


def setup_mpi(
    *,
    name: Ostrlike = None,
    version: str = 'v1',
    mpi: Ostrlike = None,
    args: Ostrlike = None,
    entrypoint: Ostrlike = None,
    condition: Oboolstr = None,
    working_directory: Ostrlike = None,
    shell: Ostr = None,
    id: Ostr = None,  # noqa: A002
    env: Mapping[str, StringLike] | None = None,
    continue_on_error: Oboollike = None,
    timeout_minutes: Ointlike = None,
) -> Step:
    options: dict[str, object] = {
        'mpi': validate_choice('mpi', mpi, ['mpich', 'openmpi', 'intelmpi', 'msmpi']),
    }

    options = {key: value for key, value in options.items() if value is not None}

    mpi_names = {
        'mpich': 'MPICH',
        'openmpi': 'Open MPI',
        'intelmpi': 'Intel MPI',
        'msmpi': 'Microsoft MPI',
    }

    if name is None:
        mpi_str = check_string(options.get('mpi'))
        if mpi_str:
            name = f'Setup {mpi_names[mpi_str]}'
        else:
            name = 'Setup MPI'

    return action(
        name,
        'mpi4py/setup-mpi',
        ref=version,
        with_opts=options or None,
        args=args,
        entrypoint=entrypoint,
        condition=condition,
        working_directory=working_directory,
        shell=shell,
        id=id,
        env=env,
        continue_on_error=continue_on_error,
        timeout_minutes=timeout_minutes,
    )



from __future__ import annotations
from typing import Union

from ..expressions import BooleanExpression, NumberExpression, StringExpression

Ostr = Union[str, None]
Obool = Union[bool, None]
Oint = Union[int, None]
StringLike = Union[str, StringExpression]
BoolLike = Union[bool, BooleanExpression]
IntLike = Union[int, NumberExpression]
Ostrlike = Union[StringLike, None]
Oboolstr = Union[BooleanExpression, str, None]
Oboollike = Union[BoolLike, None]
Ointlike = Union[IntLike, None]
StringOrBoolLike = Union[StringLike, BoolLike]

__all__ = [
    'Ostr',
    'Obool',
    'Oint',
    'StringLike',
    'BoolLike',
    'IntLike',
    'Ostrlike',
    'Oboolstr',
    'Oboollike',
    'Ointlike',
    'StringOrBoolLike',
]



from __future__ import annotations
from yamloom.actions.utils import validate_choice, check_string

from typing import TYPE_CHECKING

from ..._yamloom import Step
from ..._yamloom import action
from ..types import (
    Oboollike,
    Oboolstr,
    Ointlike,
    Ostr,
    Ostrlike,
    StringLike,
)

if TYPE_CHECKING:
    from collections.abc import Mapping

WARN_RETENTION_DAYS: int = 90
MAX_COMPRESSION_LEVEL: int = 9

__all__ = ['download_artifact', 'upload_artifact', 'upload_artifact_merge']


def upload_artifact(
    *,
    path: StringLike,
    name: Ostrlike = None,
    version: str = 'v6',
    artifact_name: Ostrlike = None,
    if_no_files_found: Ostrlike = None,
    retention_days: Ointlike = None,
    compression_level: Ointlike = None,
    overwrite: Oboollike = None,
    include_hidden_files: Oboollike = None,
    args: Ostrlike = None,
    entrypoint: Ostrlike = None,
    condition: Oboolstr = None,
    working_directory: Ostrlike = None,
    shell: Ostr = None,
    id: Ostr = None,  # noqa: A002
    env: Mapping[str, StringLike] | None = None,
    continue_on_error: Oboollike = None,
    timeout_minutes: Ointlike = None,
) -> Step:
    options: dict[str, object] = {
        'path': path,
        'name': artifact_name,
        'if_no_files_found': validate_choice(
            'if_no_files_found', if_no_files_found, ['warn', 'error', 'ignore']
        ),
        'retention-days': retention_days,
        'compression-level': compression_level,
        'overwrite': overwrite,
        'include-hidden-files': include_hidden_files,
    }

    if retention_days is not None:
        if isinstance(retention_days, int) and not isinstance(retention_days, bool):
            if retention_days < 1:
                msg = 'retention days must be > 0'
                raise ValueError(msg)
            if retention_days > WARN_RETENTION_DAYS:
                print(
                    f'Warning: retention days should be <= {WARN_RETENTION_DAYS} unless a higher limit is made in the repository settings!'
                )
        options['retention-days'] = retention_days

    if compression_level is not None:
        if (
            isinstance(compression_level, int)
            and not isinstance(compression_level, bool)
        ) and (compression_level < 0 or compression_level > MAX_COMPRESSION_LEVEL):
            msg = f'compression level must be in the range 0-{MAX_COMPRESSION_LEVEL}'
            raise ValueError(msg)
        options['compression-level'] = compression_level

    options = {key: value for key, value in options.items() if value is not None}

    if name is None:
        artifact_str = check_string(options.get('artifact_name'))
        if artifact_str:
            name = f'Upload {artifact_str}'
        else:
            name = 'Upload Artifact'

    return action(
        name,
        'actions/upload-artifact',
        ref=version,
        with_opts=options or None,
        args=args,
        entrypoint=entrypoint,
        condition=condition,
        working_directory=working_directory,
        shell=shell,
        id=id,
        env=env,
        continue_on_error=continue_on_error,
        timeout_minutes=timeout_minutes,
    )


def upload_artifact_merge(
    *,
    pattern: Ostrlike = None,
    name: Ostrlike = None,
    version: str = 'v6',
    artifact_name: Ostrlike = None,
    separate_directories: Oboollike = None,
    delete_merged: Oboollike = None,
    retention_days: Ointlike = None,
    compression_level: Ointlike = None,
    args: Ostrlike = None,
    entrypoint: Ostrlike = None,
    condition: Oboolstr = None,
    working_directory: Ostrlike = None,
    shell: Ostr = None,
    id: Ostr = None,  # noqa: A002
    env: Mapping[str, StringLike] | None = None,
    continue_on_error: Oboollike = None,
    timeout_minutes: Ointlike = None,
) -> Step:
    options: dict[str, object] = {
        'name': artifact_name,
        'pattern': pattern,
        'separate-directories': separate_directories,
        'delete-merged': delete_merged,
        'retention-days': retention_days,
        'compression-level': compression_level,
    }

    if retention_days is not None:
        if isinstance(retention_days, int) and not isinstance(retention_days, bool):
            if retention_days < 1:
                msg = 'retention days must be > 0'
                raise ValueError(msg)
            if retention_days > WARN_RETENTION_DAYS:
                print(
                    f'Warning: retention days should be <= {WARN_RETENTION_DAYS} unless a higher limit is made in the repository settings!'
                )
        options['retention-days'] = retention_days

    if compression_level is not None:
        if (
            isinstance(compression_level, int)
            and not isinstance(compression_level, bool)
        ) and (compression_level < 0 or compression_level > MAX_COMPRESSION_LEVEL):
            msg = f'compression level must be in the range 0-{MAX_COMPRESSION_LEVEL}'
            raise ValueError(msg)
        options['compression-level'] = compression_level

    options = {key: value for key, value in options.items() if value is not None}

    if name is None:
        artifact_str = check_string(options.get('artifact_name'))
        if artifact_str:
            name = f'Upload (merged) {artifact_str}'
        else:
            name = 'Upload (merged) Artifact'

    return action(
        name,
        'actions/upload-artifact/merge',
        ref=version,
        with_opts=options or None,
        args=args,
        entrypoint=entrypoint,
        condition=condition,
        working_directory=working_directory,
        shell=shell,
        id=id,
        env=env,
        continue_on_error=continue_on_error,
        timeout_minutes=timeout_minutes,
    )


def download_artifact(
    *,
    path: Ostrlike = None,
    name: Ostrlike = None,
    version: str = 'v7',
    artifact_name: Ostrlike = None,
    artifact_ids: list[StringLike] | None = None,
    pattern: Ostrlike = None,
    merge_multiple: Oboollike = None,
    github_token: Ostrlike = None,
    repository: Ostrlike = None,
    run_id: Ostrlike = None,
    args: Ostrlike = None,
    entrypoint: Ostrlike = None,
    condition: Oboolstr = None,
    working_directory: Ostrlike = None,
    shell: Ostr = None,
    id: Ostr = None,  # noqa: A002
    env: Mapping[str, StringLike] | None = None,
    continue_on_error: Oboollike = None,
    timeout_minutes: Ointlike = None,
) -> Step:
    options: dict[str, object] = {
        'name': artifact_name,
        'artifact-ids': ','.join(str(s) for s in artifact_ids)
        if artifact_ids is not None
        else None,
        'pattern': pattern,
        'path': path,
        'merge-multiple': merge_multiple,
        'github-token': github_token,
        'repository': repository,
        'run-id': run_id,
    }
    options = {key: value for key, value in options.items() if value is not None}

    if name is None:
        artifact_str = check_string(options.get('artifact_name'))
        if artifact_str:
            name = f'Download {artifact_str}'
        else:
            name = 'Download Artifact'

    return action(
        name,
        'actions/download-artifact',
        ref=version,
        with_opts=options or None,
        args=args,
        entrypoint=entrypoint,
        condition=condition,
        working_directory=working_directory,
        shell=shell,
        id=id,
        env=env,
        continue_on_error=continue_on_error,
        timeout_minutes=timeout_minutes,
    )



from pathlib import Path
from collections.abc import Mapping
from types import ModuleType
from typing import Any, Literal

from typing_extensions import TypeAlias

from .expressions import (
    BooleanExpression,
    NumberExpression,
    StringExpression,
)

Ostr: TypeAlias = str | None
Obool: TypeAlias = bool | None
Oint: TypeAlias = int | None
StringLike: TypeAlias = str | StringExpression
BoolLike: TypeAlias = bool | BooleanExpression
IntLike: TypeAlias = int | NumberExpression
Ostrlike: TypeAlias = StringLike | None
Oboolstr: TypeAlias = BooleanExpression | str | None
Oboollike: TypeAlias = BoolLike | None
Ointlike: TypeAlias = IntLike | None
StringOrBoolLike: TypeAlias = StringLike | BoolLike

expressions: ModuleType

class Step: ...

def script(
    name: StringLike,
    *script: StringLike,
    condition: Oboolstr = None,
    working_directory: Ostrlike = None,
    shell: Ostr = None,
    id: Ostr = None,  # noqa: A002
    env: Mapping[str, StringLike] | None = None,
    continue_on_error: Oboollike = None,
    timeout_minutes: Ointlike = None,
) -> Step: ...
def action(
    name: StringLike,
    action: str,
    *,
    ref: Ostr = None,
    with_opts: Mapping | None = None,
    args: Ostrlike = None,
    entrypoint: Ostrlike = None,
    condition: Oboolstr = None,
    working_directory: Ostrlike = None,
    shell: Ostr = None,
    id: Ostr = None,  # noqa: A002
    env: Mapping[str, StringLike] | None = None,
    continue_on_error: Oboollike = None,
    timeout_minutes: Ointlike = None,
) -> Step: ...

RW: TypeAlias = Literal['read', 'write', 'none'] | None
RO: TypeAlias = Literal['read', 'none'] | None
WO: TypeAlias = Literal['write', 'none'] | None

class Permissions:
    def __init__(
        self,
        *,
        actions: RW = None,
        artifact_metadata: RW = None,
        attestations: RW = None,
        checks: RW = None,
        contents: RW = None,
        deployments: RW = None,
        id_token: WO = None,
        issues: RW = None,
        models: RO = None,
        discussions: RW = None,
        packages: RW = None,
        pages: RW = None,
        pull_requests: RW = None,
        security_events: RW = None,
        statuses: RW = None,
    ) -> None: ...
    @staticmethod
    def none() -> Permissions: ...
    @staticmethod
    def read_all() -> Permissions: ...
    @staticmethod
    def write_all() -> Permissions: ...

class RunsOnSpec:
    def __init__(self, group: StringLike, labels: StringLike) -> None: ...
    @staticmethod
    def group(group: StringLike) -> RunsOnSpec: ...
    @staticmethod
    def labels(labels: StringLike) -> RunsOnSpec: ...

class RunsOn:
    def __init__(self, *args: StringLike) -> None: ...
    @staticmethod
    def spec(spec: RunsOnSpec) -> RunsOn: ...

class Environment:
    def __init__(self, name: StringLike, url: Ostrlike = None) -> None: ...

class Concurrency:
    def __init__(
        self, group: StringLike, *, cancel_in_progress: Oboollike = None
    ) -> None: ...

class RunDefaults:
    def __init__(
        self, *, shell: Ostrlike = None, working_directory: Ostrlike = None
    ) -> None: ...

class Defaults:
    def __init__(
        self,
        *,
        defaults: Mapping[str, str] | None = None,
        run_defaults: RunDefaults | None = None,
    ) -> None: ...

class Matrix:
    def __init__(
        self,
        *,
        include: list | None = None,
        exclude: list | None = None,
        **matrix: Any,
    ) -> None: ...

class Strategy:
    def __init__(
        self,
        *,
        matrix: Matrix | None = None,
        fast_fail: Oboollike = None,
        max_parallel: Ointlike = None,
    ) -> None: ...

class Credentials:
    def __init__(self, username: StringLike, password: StringLike) -> None: ...

class Container:
    def __init__(
        self,
        image: StringLike,
        *,
        credentials: Credentials | None = None,
        env: Mapping[str, StringLike] | None = None,
        ports: list[IntLike] | None = None,
        volumes: list[StringLike] | None = None,
        options: Ostrlike = None,
    ) -> None: ...

class JobSecrets:
    def __init__(self, secrets: Mapping[str, StringLike]) -> None: ...
    @staticmethod
    def inherit() -> JobSecrets: ...

class Job:
    def __init__(
        self,
        steps: list[Step],
        *,
        name: Ostrlike = None,
        permissions: Permissions | None = None,
        needs: list[str] | None = None,
        condition: Oboolstr = None,
        runs_on: RunsOnSpec | list[StringLike] | StringLike | None = None,
        snapshot: Ostr = None,
        environment: Environment | None = None,
        concurrency: Concurrency | None = None,
        outputs: Mapping[str, StringLike] | None = None,
        env: Mapping[str, StringLike] | None = None,
        defaults: Defaults | None = None,
        timeout_minutes: Oint = None,
        strategy: Strategy | None = None,
        continue_on_error: StringOrBoolLike | None = None,
        container: Container | None = None,
        services: Mapping[str, Container] | None = None,
        uses: Ostr = None,
        with_opts: Mapping | None = None,
        secrets: JobSecrets | None = None,
    ) -> None: ...

class BranchProtectionRuleEvent:
    def __init__(
        self, *, created: bool = False, edited: bool = False, deleted: bool = False
    ) -> None: ...

class CheckRunEvent:
    def __init__(
        self,
        *,
        created: bool = False,
        rerequested: bool = False,
        completed: bool = False,
        requested_action: bool = False,
    ) -> None: ...

class CheckSuiteEvent:
    def __init__(self, *, created: bool = False) -> None: ...

class DiscussionEvent:
    def __init__(
        self,
        *,
        created: bool = False,
        edited: bool = False,
        deleted: bool = False,
        transferred: bool = False,
        pinned: bool = False,
        unpinned: bool = False,
        labeled: bool = False,
        unlabeled: bool = False,
        locked: bool = False,
        unlocked: bool = False,
        category_changed: bool = False,
        answered: bool = False,
        unanswered: bool = False,
    ) -> None: ...

class DiscussionCommentEvent:
    def __init__(
        self, *, created: bool = False, edited: bool = False, deleted: bool = False
    ) -> None: ...

class ImageVersionEvent:
    def __init__(
        self,
        *,
        names: list[str] | None = None,
        versions: list[str] | None = None,
    ) -> None: ...

class IssueCommentEvent:
    def __init__(
        self, *, created: bool = False, edited: bool = False, deleted: bool = False
    ) -> None: ...

class IssuesEvent:
    def __init__(
        self,
        *,
        created: bool = False,
        edited: bool = False,
        deleted: bool = False,
        transferred: bool = False,
        pinned: bool = False,
        unpinned: bool = False,
        closed: bool = False,
        reopened: bool = False,
        assigned: bool = False,
        unassigned: bool = False,
        labeled: bool = False,
        unlabeled: bool = False,
        locked: bool = False,
        unlocked: bool = False,
        milestoned: bool = False,
        demilestoned: bool = False,
        typed: bool = False,
        untyped: bool = False,
    ) -> None: ...

class LabelEvent:
    def __init__(
        self, *, created: bool = False, edited: bool = False, deleted: bool = False
    ) -> None: ...

class MergeGroupEvent:
    def __init__(self, *, checks_requested: bool = False) -> None: ...

class MilestoneEvent:
    def __init__(
        self,
        *,
        created: bool = False,
        closed: bool = False,
        opened: bool = False,
        edited: bool = False,
        deleted: bool = False,
    ) -> None: ...

class PullRequestEvent:
    def __init__(
        self,
        *,
        branches: list[str] | None = None,
        branches_ignore: list[str] | None = None,
        paths: list[str] | None = None,
        paths_ignore: list[str] | None = None,
        assigned: bool = False,
        unassigned: bool = False,
        labeled: bool = False,
        unlabeled: bool = False,
        opened: bool = False,
        edited: bool = False,
        closed: bool = False,
        reopened: bool = False,
        synchronize: bool = False,
        converted_to_draft: bool = False,
        locked: bool = False,
        unlocked: bool = False,
        enqueued: bool = False,
        dequeued: bool = False,
        milestoned: bool = False,
        demilestoned: bool = False,
        ready_for_review: bool = False,
        review_requested: bool = False,
        review_request_removed: bool = False,
        auto_merge_enabled: bool = False,
        auto_merge_disabled: bool = False,
    ) -> None: ...

class PullRequestReviewEvent:
    def __init__(
        self, *, submitted: bool = False, edited: bool = False, dismissed: bool = False
    ) -> None: ...

class PullRequestReviewCommentEvent:
    def __init__(
        self, *, created: bool = False, edited: bool = False, deleted: bool = False
    ) -> None: ...

class PushEvent:
    def __init__(
        self,
        *,
        branches: list[str] | None = None,
        branches_ignore: list[str] | None = None,
        tags: list[str] | None = None,
        tags_ignore: list[str] | None = None,
        paths: list[str] | None = None,
        paths_ignore: list[str] | None = None,
    ) -> None: ...

class RegistryPackageEvent:
    def __init__(self, *, published: bool = False, updated: bool = False) -> None: ...

class ReleaseEvent:
    def __init__(
        self,
        *,
        published: bool = False,
        unpublished: bool = False,
        created: bool = False,
        edited: bool = False,
        deleted: bool = False,
        prereleased: bool = False,
        released: bool = False,
    ) -> None: ...

class RepositoryDispatchEvent:
    def __init__(self, *, types: list[str] | None = None) -> None: ...

class Minute:
    def __init__(self, minute: int | list[int]) -> None: ...
    @staticmethod
    def between(start: int, end: int) -> Minute: ...
    @staticmethod
    def every(interval: int, *, start: Oint = None) -> Minute: ...

class Hour:
    def __init__(self, minute: int | list[int]) -> None: ...
    @staticmethod
    def between(start: int, end: int) -> Hour: ...
    @staticmethod
    def every(interval: int, *, start: Oint = None) -> Hour: ...

class Day:
    def __init__(self, minute: int | list[int]) -> None: ...
    @staticmethod
    def between(start: int, end: int) -> Day: ...
    @staticmethod
    def every(interval: int, *, start: Oint = None) -> Day: ...

class Month:
    def __init__(self, minute: int | list[int]) -> None: ...
    @staticmethod
    def between(start: int, end: int) -> Month: ...
    @staticmethod
    def every(interval: int, *, start: Oint = None) -> Month: ...

class DayOfWeek:
    def __init__(self, minute: int | list[int]) -> None: ...
    @staticmethod
    def between(start: int, end: int) -> DayOfWeek: ...
    @staticmethod
    def every(interval: int, *, start: Oint = None) -> DayOfWeek: ...

class Cron:
    def __init__(
        self,
        *,
        minute: Minute | None = None,
        hour: Hour | None = None,
        day: Day | None = None,
        month: Month | None = None,
        day_of_week: DayOfWeek | None = None,
    ) -> None: ...

class ScheduleEvent:
    def __init__(self, *, crons: list[Cron] | None = None) -> None: ...

class WatchEvent:
    def __init__(self, *, started: bool = False) -> None: ...

class WorkflowInput:
    @staticmethod
    def boolean(
        *, description: Ostr = None, default: Oboollike = None, required: Obool = None
    ) -> WorkflowInput: ...
    @staticmethod
    def number(
        *, description: Ostr = None, default: Ointlike = None, required: Obool = None
    ) -> WorkflowInput: ...
    @staticmethod
    def string(
        *, description: Ostr = None, default: Ostrlike = None, required: Obool = None
    ) -> WorkflowInput: ...

class WorkflowOutput:
    def __init__(self, value: StringLike, *, description: Ostr = None) -> None: ...

class WorkflowSecret:
    def __init__(self, *, description: Ostr = None, required: Obool = None) -> None: ...

class WorkflowCallEvent:
    def __init__(
        self,
        *,
        inputs: Mapping[str, WorkflowInput] | None = None,
        outputs: Mapping[str, WorkflowOutput] | None = None,
        secrets: Mapping[str, WorkflowSecret] | None = None,
    ) -> None: ...

class WorkflowDispatchInput:
    @staticmethod
    def boolean(
        *, description: Ostr = None, default: Obool = None, required: Obool = None
    ) -> WorkflowDispatchInput: ...
    @staticmethod
    def choice(
        options: list[str],
        *,
        description: Ostr = None,
        default: Ostr = None,
        required: Obool = None,
    ) -> WorkflowDispatchInput: ...
    @staticmethod
    def number(
        *, description: Ostr = None, default: Oint = None, required: Obool = None
    ) -> WorkflowDispatchInput: ...
    @staticmethod
    def environment(
        *, description: Ostr = None, required: Obool = None
    ) -> WorkflowDispatchInput: ...
    @staticmethod
    def string(
        *, description: Ostr = None, default: Ostr = None, required: Obool = None
    ) -> WorkflowDispatchInput: ...

class WorkflowDispatchEvent:
    def __init__(
        self, *, inputs: Mapping[str, WorkflowDispatchInput] | None = None
    ) -> None: ...

class WorkflowRunEvent:
    def __init__(
        self,
        *,
        workflows: list[str] | None = None,
        completed: bool = False,
        requested: bool = False,
        in_progress: bool = False,
        branches: list[str] | None = None,
        branches_ignore: list[str] | None = None,
    ) -> None: ...

class Events:
    def __init__(
        self,
        branch_protection_rule: BranchProtectionRuleEvent | None = None,
        check_run: CheckRunEvent | None = None,
        check_suite: CheckSuiteEvent | None = None,
        create: bool = False,
        delete: bool = False,
        deployment: bool = False,
        deployment_status: bool = False,
        discussion: DiscussionEvent | None = None,
        discussion_comment: DiscussionCommentEvent | None = None,
        fork: bool = False,
        gollum: bool = False,
        image_version: ImageVersionEvent | None = None,
        issue_comment: IssueCommentEvent | None = None,
        issues: IssuesEvent | None = None,
        label: LabelEvent | None = None,
        merge_group: MergeGroupEvent | None = None,
        milestone: MilestoneEvent | None = None,
        page_build: bool = False,
        public: bool = False,
        pull_request: PullRequestEvent | None = None,
        pull_request_review: PullRequestReviewEvent | None = None,
        pull_request_review_comment: PullRequestReviewCommentEvent | None = None,
        pull_request_target: PullRequestEvent | None = None,
        push: PushEvent | None = None,
        registry_package: RegistryPackageEvent | None = None,
        release: ReleaseEvent | None = None,
        schedule: ScheduleEvent | None = None,
        status: bool = False,
        watch: WatchEvent | None = None,
        workflow_call: WorkflowCallEvent | None = None,
        workflow_dispatch: WorkflowDispatchEvent | None = None,
        workflow_run: WorkflowRunEvent | None = None,
    ) -> None: ...

class Workflow:
    def __init__(
        self,
        *,
        jobs: Mapping[str, Job],
        on: Events,
        name: Ostr = None,
        run_name: Ostrlike = None,
        permissions: Permissions | None = None,
        env: Mapping[str, StringLike] | None = None,
        defaults: Defaults | None = None,
        concurrency: Concurrency | None = None,
    ) -> None: ...
    def dump(self, path: Path | str, *, overwrite: bool = True) -> None: ...

__all__ = [
    'BranchProtectionRuleEvent',
    'CheckRunEvent',
    'CheckSuiteEvent',
    'Concurrency',
    'Container',
    'Credentials',
    'Cron',
    'Day',
    'DayOfWeek',
    'Defaults',
    'DiscussionCommentEvent',
    'DiscussionEvent',
    'Environment',
    'Events',
    'Hour',
    'ImageVersionEvent',
    'IssueCommentEvent',
    'IssuesEvent',
    'Job',
    'JobSecrets',
    'LabelEvent',
    'Matrix',
    'MergeGroupEvent',
    'MilestoneEvent',
    'Minute',
    'Month',
    'Permissions',
    'PullRequestEvent',
    'PullRequestReviewCommentEvent',
    'PullRequestReviewEvent',
    'PushEvent',
    'RegistryPackageEvent',
    'ReleaseEvent',
    'RepositoryDispatchEvent',
    'RunDefaults',
    'RunsOn',
    'RunsOnSpec',
    'ScheduleEvent',
    'Step',
    'Strategy',
    'WatchEvent',
    'Workflow',
    'WorkflowCallEvent',
    'WorkflowDispatchEvent',
    'WorkflowDispatchInput',
    'WorkflowInput',
    'WorkflowOutput',
    'WorkflowRunEvent',
    'WorkflowSecret',
    'action',
    'script',
]



from yamloom.actions.types import Ostrlike
from collections.abc import Sequence


def validate_choice(
    option_name: str, option_value: Ostrlike, choices: Sequence[str]
) -> Ostrlike:
    if option_value is not None:
        if isinstance(option_value, str):
            lowered = option_value.lower()
            if option_value not in choices:
                quoted_choices = [f"'{c}'" for c in choices]
                if len(choices) > 2:
                    choices_str = (
                        f'{", ".join(quoted_choices[:-1])}, or {quoted_choices[-1]}'
                    )
                else:
                    choices_str = ' or '.join(quoted_choices)
                msg = f"'{option_name}' must be {choices_str}"
                raise ValueError(msg)
            return lowered
        else:
            return option_value
    else:
        return None


def check_string(s: object | None) -> str | None:
    if isinstance(s, str):
        if '${{' not in s:
            return s
    return None



from __future__ import annotations

from typing import TYPE_CHECKING

from ..._yamloom import Step
from ..._yamloom import action
from ..types import (
    Oboollike,
    Oboolstr,
    Ointlike,
    Ostr,
    Ostrlike,
    StringLike,
)

if TYPE_CHECKING:
    from collections.abc import Mapping

__all__ = ['setup_go']


def setup_go(
    *,
    name: Ostrlike = None,
    version: str = 'v6',
    go_version: Ostrlike = None,
    go_version_file: Ostrlike = None,
    check_latest: Oboollike = None,
    architecture: Ostrlike = None,
    token: Ostrlike = None,
    cache: Oboollike = None,
    cache_dependency_path: Ostrlike = None,
    args: Ostrlike = None,
    entrypoint: Ostrlike = None,
    condition: Oboolstr = None,
    working_directory: Ostrlike = None,
    shell: Ostr = None,
    id: Ostr = None,  # noqa: A002
    env: Mapping[str, StringLike] | None = None,
    continue_on_error: Oboollike = None,
    timeout_minutes: Ointlike = None,
) -> Step:
    options: dict[str, object] = {
        'go-version': go_version,
        'go-version-file': go_version_file,
        'check-latest': check_latest,
        'architecture': architecture,
        'token': token,
        'cache': cache,
        'cache-dependency-path': cache_dependency_path,
    }
    options = {key: value for key, value in options.items() if value is not None}

    if name is None:
        name = 'Setup Go'

    return action(
        name,
        'actions/setup-go',
        ref=version,
        with_opts=options or None,
        args=args,
        entrypoint=entrypoint,
        condition=condition,
        working_directory=working_directory,
        shell=shell,
        id=id,
        env=env,
        continue_on_error=continue_on_error,
        timeout_minutes=timeout_minutes,
    )



from __future__ import annotations
from yamloom.actions.utils import validate_choice

from typing import TYPE_CHECKING

from ..._yamloom import Step
from ..._yamloom import action
from ..types import (
    Oboollike,
    Oboolstr,
    Ointlike,
    Ostr,
    Ostrlike,
    StringLike,
    StringOrBoolLike,
)

if TYPE_CHECKING:
    from collections.abc import Mapping

__all__ = ['setup_python', 'setup_uv']


def setup_python(
    *,
    name: Ostrlike = None,
    version: str = 'v6',
    python_version: StringLike | None = None,
    python_version_file: Ostrlike = None,
    check_latest: Oboollike = None,
    architecture: Ostrlike = None,
    token: Ostrlike = None,
    cache: Ostrlike = None,
    package_manager_cache: Oboollike = None,
    cache_dependency_path: Ostrlike = None,
    update_environment: Oboollike = None,
    allow_prereleases: Oboollike = None,
    freethreaded: Oboollike = None,
    pip_versions: Ostrlike = None,
    pip_install: Ostrlike = None,
    args: Ostrlike = None,
    entrypoint: Ostrlike = None,
    condition: Oboolstr = None,
    working_directory: Ostrlike = None,
    shell: Ostr = None,
    id: Ostr = None,  # noqa: A002
    env: Mapping[str, StringLike] | None = None,
    continue_on_error: Oboollike = None,
    timeout_minutes: Ointlike = None,
) -> Step:
    options: dict[str, object] = {
        'python-version': python_version,
        'python-version-file': python_version_file,
        'check-latest': check_latest,
        'architecture': architecture,
        'token': token,
        'cache': cache,
        'package-manager-cache': package_manager_cache,
        'cache-dependency-path': cache_dependency_path,
        'update-environment': update_environment,
        'allow-prereleases': allow_prereleases,
        'freethreaded': freethreaded,
        'pip-versions': pip_versions,
        'pip-install': pip_install,
    }

    if cache is not None:
        if isinstance(cache, str):
            lowered = cache.lower()
            if lowered not in {'pip', 'pipenv', 'poetry'}:
                msg = "'cache' must be 'pip', 'pipenv' or 'poetry'"
                raise ValueError(msg)
            options['cache'] = lowered
        else:
            options['cache'] = cache

    options = {key: value for key, value in options.items() if value is not None}

    if name is None:
        name = 'Setup Python'

    return action(
        name,
        'actions/setup-python',
        ref=version,
        with_opts=options or None,
        args=args,
        entrypoint=entrypoint,
        condition=condition,
        working_directory=working_directory,
        shell=shell,
        id=id,
        env=env,
        continue_on_error=continue_on_error,
        timeout_minutes=timeout_minutes,
    )


def setup_uv(
    *,
    name: Ostrlike = None,
    version: str = 'v7',
    uv_version: Ostrlike = None,
    uv_version_file: Ostrlike = None,
    resolution_strategy: Ostrlike = None,
    python_version: Ostrlike = None,
    activate_environment: Oboollike = None,
    uv_working_directory: Ostrlike = None,
    checksum: Ostrlike = None,
    github_token: Ostrlike = None,
    enable_cache: StringOrBoolLike | None = None,
    cache_dependency_glob: list[StringLike] | None = None,
    restore_cache: Oboollike = None,
    save_cache: Oboollike = None,
    cache_suffix: Ostrlike = None,
    cache_local_path: Ostrlike = None,
    prune_cache: Oboollike = None,
    cache_python: Oboollike = None,
    ignore_nothing_to_cache: Oboollike = None,
    ignore_empty_workdir: Oboollike = None,
    tool_dir: Ostrlike = None,
    tool_bin_dir: Ostrlike = None,
    manifest_file: Ostrlike = None,
    add_problem_matchers: Oboollike = None,
    args: Ostrlike = None,
    entrypoint: Ostrlike = None,
    condition: Oboolstr = None,
    working_directory: Ostrlike = None,
    shell: Ostr = None,
    id: Ostr = None,  # noqa: A002
    env: Mapping[str, StringLike] | None = None,
    continue_on_error: Oboollike = None,
    timeout_minutes: Ointlike = None,
) -> Step:
    options: dict[str, object] = {
        'version': uv_version,
        'version-file': uv_version_file,
        'resolution-strategy': validate_choice(
            'resolution_strategy', resolution_strategy, ['highest', 'lowest']
        ),
        'python-version': python_version,
        'activate-environment': activate_environment,
        'working-directory': uv_working_directory,
        'checksum': checksum,
        'github-token': github_token,
        'enable-cache': enable_cache,
        'cache-dependency-glob': '\n'.join(str(s) for s in cache_dependency_glob)
        if cache_dependency_glob is not None
        else None,
        'restore-cache': restore_cache,
        'save-cache': save_cache,
        'cache-suffix': cache_suffix,
        'cache-local-path': cache_local_path,
        'prune-cache': prune_cache,
        'cache-python': cache_python,
        'ignore-nothing-to-cache': ignore_nothing_to_cache,
        'ignore-empty-workdir': ignore_empty_workdir,
        'tool-dir': tool_dir,
        'tool-bin-dir': tool_bin_dir,
        'manifest-file': manifest_file,
        'add-problem-matchers': add_problem_matchers,
    }

    if enable_cache is not None:
        if isinstance(enable_cache, str):
            if enable_cache.lower() != 'auto':
                msg = "'enable_cache' must be 'auto', true or false"
                raise ValueError(msg)
            options['enable-cache'] = 'auto'
        elif isinstance(enable_cache, bool):
            options['enable-cache'] = enable_cache
        else:
            options['enable-cache'] = enable_cache

    options = {key: value for key, value in options.items() if value is not None}

    if name is None:
        name = 'Setup uv'

    return action(
        name,
        'astral-sh/setup-uv',
        ref=version,
        with_opts=options or None,
        args=args,
        entrypoint=entrypoint,
        condition=condition,
        working_directory=working_directory,
        shell=shell,
        id=id,
        env=env,
        continue_on_error=continue_on_error,
        timeout_minutes=timeout_minutes,
    )



from __future__ import annotations
from yamloom.actions.utils import check_string

from typing import TYPE_CHECKING

from ..._yamloom import Step
from ..._yamloom import action
from ..types import (
    Obool,
    Oboollike,
    Oboolstr,
    Ointlike,
    Ostr,
    Ostrlike,
    StringLike,
)

if TYPE_CHECKING:
    from collections.abc import Mapping

__all__ = ['release']


def release(
    *,
    name: Ostrlike = None,
    version: str = 'v2',
    body: Ostr = None,
    body_path: Ostr = None,
    draft: Obool = None,
    prerelease: Obool = None,
    preserve_order: Obool = None,
    files: Ostr = None,
    overwrite_files: Obool = None,
    release_name: Ostr = None,
    tag_name: Ostr = None,
    fail_on_unmatched_files: Obool = None,
    repository: Ostr = None,
    target_commitish: Ostr = None,
    token: Ostr = None,
    discussion_category_name: Ostr = None,
    generate_release_notes: Obool = None,
    append_body: Obool = None,
    make_latest: Ostr = None,
    args: Ostrlike = None,
    entrypoint: Ostrlike = None,
    condition: Oboolstr = None,
    working_directory: Ostrlike = None,
    shell: Ostr = None,
    id: Ostr = None,  # noqa: A002
    env: Mapping[str, StringLike] | None = None,
    continue_on_error: Oboollike = None,
    timeout_minutes: Ointlike = None,
) -> Step:
    options: dict[str, object] = {
        'body': body,
        'body_path': body_path,
        'draft': draft,
        'prerelease': prerelease,
        'preserve_order': preserve_order,
        'files': files,
        'overwrite_files': overwrite_files,
        'name': release_name,
        'tag_name': tag_name,
        'fail_on_unmatched_files': fail_on_unmatched_files,
        'repository': repository,
        'target_commitish': target_commitish,
        'token': token,
        'discussion_category_name': discussion_category_name,
        'generate_release_notes': generate_release_notes,
        'append_body': append_body,
        'make_latest': make_latest,
    }
    options = {key: value for key, value in options.items() if value is not None}

    if name is None:
        repository_str = check_string(options.get('repository'))
        if repository_str:
            name = f"Make Release for '{repository_str}'"
        else:
            name = 'Make Release'

    return action(
        name,
        'softprops/action-gh-release',
        ref=version,
        with_opts=options or None,
        args=args,
        entrypoint=entrypoint,
        condition=condition,
        working_directory=working_directory,
        shell=shell,
        id=id,
        env=env,
        continue_on_error=continue_on_error,
        timeout_minutes=timeout_minutes,
    )



from __future__ import annotations
from yamloom.actions.utils import validate_choice

from typing import TYPE_CHECKING

from ..._yamloom import Step
from ..._yamloom import action
from ..types import (
    Oboollike,
    Oboolstr,
    Ointlike,
    Ostr,
    Ostrlike,
    StringLike,
)

if TYPE_CHECKING:
    from collections.abc import Mapping

__all__ = ['setup_dotnet']


def setup_dotnet(
    *,
    name: Ostrlike = None,
    version: str = 'v5',
    dotnet_version: Ostrlike = None,
    dotnet_quality: Ostrlike = None,
    global_json_file: Ostrlike = None,
    source_url: Ostrlike = None,
    owner: Ostrlike = None,
    config_file: Ostrlike = None,
    cache: Oboollike = None,
    cache_dependency_path: Ostrlike | list[StringLike] = None,
    args: Ostrlike = None,
    entrypoint: Ostrlike = None,
    condition: Oboolstr = None,
    working_directory: Ostrlike = None,
    shell: Ostr = None,
    id: Ostr = None,  # noqa: A002
    env: Mapping[str, StringLike] | None = None,
    continue_on_error: Oboollike = None,
    timeout_minutes: Ointlike = None,
) -> Step:
    options: dict[str, object] = {
        'dotnet-version': dotnet_version,
        'dotnet-quality': validate_choice(
            'dotnet-quality',
            dotnet_quality,
            ['daily', 'signed', 'validated', 'preview', 'ga'],
        ),
        'global-json-file': global_json_file,
        'source-url': source_url,
        'owner': owner,
        'config-file': config_file,
        'cache': cache,
        'cache-dependency-path': cache_dependency_path,
    }
    options = {key: value for key, value in options.items() if value is not None}

    if name is None:
        name = 'Setup .NET'

    return action(
        name,
        'actions/setup-dotnet',
        ref=version,
        with_opts=options or None,
        args=args,
        entrypoint=entrypoint,
        condition=condition,
        working_directory=working_directory,
        shell=shell,
        id=id,
        env=env,
        continue_on_error=continue_on_error,
        timeout_minutes=timeout_minutes,
    )



from yamloom.actions.toolchains.rust import setup_rust
from dataclasses import dataclass
from yamloom.actions.github.artifacts import upload_artifact, download_artifact
from yamloom.actions.packaging.python import maturin
from yamloom.actions.toolchains.python import setup_python, setup_uv
from yamloom.actions.github.scm import checkout
from yamloom.expressions import context
from yamloom import (
    Workflow,
    Events,
    PushEvent,
    PullRequestEvent,
    WorkflowDispatchEvent,
    Permissions,
    Job,
    Matrix,
    Strategy,
    script,
    Environment,
    action,
)


@dataclass
class Target:
    runner: str
    target: str
    skip_python_versions: list[str] | None = None


DEFAULT_PYTHON_VERSIONS = [
    '3.9',
    '3.10',
    '3.11',
    '3.12',
    '3.13',
    '3.13t',
    '3.14',
    '3.14t',
    'pypy3.11',
]


def resolve_python_versions(skip: list[str] | None) -> list[str]:
    if not skip:
        return DEFAULT_PYTHON_VERSIONS
    skipped = set(skip)
    return [version for version in DEFAULT_PYTHON_VERSIONS if version not in skipped]


def create_build_job(
    job_name: str, name: str, targets: list[Target], *, needs: list[str]
) -> Job:
    def platform_entry(target: Target) -> dict[str, object]:
        entry = {
            'runner': target.runner,
            'target': target.target,
            'python_versions': resolve_python_versions(target.skip_python_versions),
        }
        python_arch = (
            ('arm64' if target.target == 'aarch64' else target.target)
            if name == 'windows'
            else None
        )
        if python_arch is not None:
            entry['python_arch'] = python_arch
        return entry

    return Job(
        [
            checkout(),
            script(
                'Write version file',
                f'printf "%s\n" {context.matrix.platform.python_versions.as_array().join(" ")} >> version.txt',
            ),
            setup_python(
                python_version_file='version.txt',
                architecture=context.matrix.platform.python_arch.as_str()
                if name == 'windows'
                else None,
            ),
            maturin(
                name='Build wheels',
                target=context.matrix.platform.target.as_str(),
                args=f'--release --out dist --interpreter {context.matrix.platform.python_versions.as_array().join(" ")}',
                sccache=~context.github.ref.startswith('refs/tags/'),
                manylinux='musllinux_1_2'
                if name == 'musllinux'
                else ('auto' if name == 'linux' else None),
            ),
            upload_artifact(
                path='dist',
                artifact_name=f'wheels-{name}-{context.matrix.platform.target}',
            ),
        ],
        runs_on=context.matrix.platform.runner.as_str(),
        strategy=Strategy(
            fast_fail=False,
            matrix=Matrix(
                platform=[platform_entry(target) for target in targets],
            ),
        ),
        needs=needs,
        condition=context.github.ref.startswith('refs/tags/')
        | (context.github.event_name == 'workflow_dispatch'),
    )


release_workflow = Workflow(
    name='Build and Release',
    on=Events(
        push=PushEvent(branches=['main'], tags=['*']),
        pull_request=PullRequestEvent(),
        workflow_dispatch=WorkflowDispatchEvent(),
    ),
    permissions=Permissions(contents='read'),
    jobs={
        'build-test-check': Job(
            [
                checkout(),
                setup_rust(components=['clippy']),
                setup_uv(python_version='3.9'),
                script('Check Rust', 'cargo clippy'),
                script('Test Rust', 'cargo test'),
                script(
                    'Setup venv',
                    'uv venv',
                    '. .venv/bin/activate',
                    'echo PATH=$PATH >> $GITHUB_ENV',
                    'uvx maturin develop --uv',
                ),
                script('Check Python', 'uvx ruff check', 'uvx ty check'),
                script('Test Python', 'uvx pytest'),
            ],
            runs_on='ubuntu-latest',
        ),
        'linux': create_build_job(
            'Build Linux Wheels',
            'linux',
            [
                Target(
                    'ubuntu-22.04',
                    target,
                )
                for target in [
                    'x86_64',
                    'x86',
                    'aarch64',
                    'armv7',
                    's390x',
                    'ppc64le',
                ]
            ],
            needs=['build-test-check'],
        ),
        'musllinux': create_build_job(
            'Build (musl) Linux Wheels',
            'musllinux',
            [
                Target(
                    'ubuntu-22.04',
                    target,
                )
                for target in [
                    'x86_64',
                    'x86',
                    'aarch64',
                    'armv7',
                ]
            ],
            needs=['build-test-check'],
        ),
        'windows': create_build_job(
            'Build Windows Wheels',
            'windows',
            [
                Target(
                    'windows-latest',
                    'x64',
                ),
                Target('windows-latest', 'x86', ['pypy3.11']),
                Target(
                    'windows-11-arm',
                    'aarch64',
                    ['3.9', '3.10', '3.11', '3.13t', '3.14t', 'pypy3.11'],
                ),
            ],
            needs=['build-test-check'],
        ),
        'macos': create_build_job(
            'Build macOS Wheels',
            'macos',
            [
                Target(
                    'macos-15-intel',
                    'x86_64',
                ),
                Target(
                    'macos-latest',
                    'aarch64',
                ),
            ],
            needs=['build-test-check'],
        ),
        'sdist': Job(
            [
                checkout(),
                maturin(name='Build sdist', command='sdist', args='--out dist'),
                upload_artifact(path='dist', artifact_name='wheels-sdist'),
            ],
            name='Build Source Distribution',
            runs_on='ubuntu-22.04',
            needs=['build-test-check'],
            condition=context.github.ref.startswith('refs/tags/')
            | (context.github.event_name == 'workflow_dispatch'),
        ),
        'release': Job(
            [
                download_artifact(),
                setup_uv(),
                script(
                    'Publish to PyPI',
                    'uv publish --trusted-publishing always wheels-*/*',
                ),
            ],
            name='Release',
            runs_on='ubuntu-22.04',
            condition=context.github.ref.startswith('refs/tags/')
            | (context.github.event_name == 'workflow_dispatch'),
            needs=['linux', 'musllinux', 'windows', 'macos', 'sdist'],
            permissions=Permissions(id_token='write', contents='write'),
            environment=Environment('pypi'),
        ),
    },
)

version_workflow = Workflow(
    name='Release Please',
    on=Events(
        push=PushEvent(
            branches=['main'],
        ),
    ),
    permissions=Permissions(contents='write', issues='write', pull_requests='write'),
    jobs={
        'release-please': Job(
            [
                action(
                    'Release Please Action',
                    'googleapis/release-please-action',
                    ref='v4',
                    with_opts={
                        'token': context.secrets.RELEASE_PLEASE,
                        'config-file': 'release-please-config.json',
                        'manifest-file': '.release-please-manifest.json',
                    },
                )
            ],
            runs_on='ubuntu-latest',
        )
    },
)


if __name__ == '__main__':
    release_workflow.dump('.github/workflows/release.yml')
    version_workflow.dump('.github/workflows/release-please.yml')



from __future__ import annotations

from typing import TYPE_CHECKING

from ..._yamloom import Step
from ..._yamloom import action
from ..types import (
    Obool,
    Oboollike,
    Oboolstr,
    Oint,
    Ointlike,
    Ostr,
    Ostrlike,
    StringLike,
)

if TYPE_CHECKING:
    from collections.abc import Mapping

__all__ = ['cache', 'cache_restore', 'cache_save']


def cache(
    *,
    key: str,
    name: Ostrlike = None,
    version: str = 'v5',
    path: list[str] | None = None,
    restore_keys: list[str] | None = None,
    enable_cross_os_archive: Obool = None,
    fail_on_cache_miss: Obool = None,
    lookup_only: Obool = None,
    segment_download_timeout_mins: Oint = None,
    args: Ostrlike = None,
    entrypoint: Ostrlike = None,
    condition: Oboolstr = None,
    working_directory: Ostrlike = None,
    shell: Ostr = None,
    id: Ostr = None,  # noqa: A002
    env: Mapping[str, StringLike] | None = None,
    continue_on_error: Oboollike = None,
    timeout_minutes: Ointlike = None,
) -> Step:
    options: dict[str, object] = {
        'key': key,
        'path': list(path) if path is not None else None,
        'restore-keys': list(restore_keys) if restore_keys is not None else None,
        'ensembleCrossOsArchive': enable_cross_os_archive,
        'fail-on-cache-miss': fail_on_cache_miss,
        'lookup-only': lookup_only,
    }
    options = {key: value for key, value in options.items() if value is not None}

    if name is None:
        name = 'Save/Restore Cache'

    if segment_download_timeout_mins is not None:
        merged_env = dict(env or {})
        merged_env['SEGMENT_DOWNLOAD_TIMEOUT_MINS'] = str(segment_download_timeout_mins)
        env = merged_env

    return action(
        name,
        'actions/cache',
        ref=version,
        with_opts=options or None,
        args=args,
        entrypoint=entrypoint,
        condition=condition,
        working_directory=working_directory,
        shell=shell,
        id=id,
        env=env,
        continue_on_error=continue_on_error,
        timeout_minutes=timeout_minutes,
    )


def cache_save(
    *,
    key: str,
    name: Ostrlike = None,
    version: str = 'v5',
    path: list[str] | None = None,
    restore_keys: list[str] | None = None,
    enable_cross_os_archive: Obool = None,
    fail_on_cache_miss: Obool = None,
    lookup_only: Obool = None,
    segment_download_timeout_mins: Oint = None,
    args: Ostrlike = None,
    entrypoint: Ostrlike = None,
    condition: Oboolstr = None,
    working_directory: Ostrlike = None,
    shell: Ostr = None,
    id: Ostr = None,  # noqa: A002
    env: Mapping[str, StringLike] | None = None,
    continue_on_error: Oboollike = None,
    timeout_minutes: Ointlike = None,
) -> Step:
    options: dict[str, object] = {
        'key': key,
        'path': list(path) if path is not None else None,
        'restore-keys': list(restore_keys) if restore_keys is not None else None,
        'ensembleCrossOsArchive': enable_cross_os_archive,
        'fail-on-cache-miss': fail_on_cache_miss,
        'lookup-only': lookup_only,
    }
    options = {key: value for key, value in options.items() if value is not None}

    if name is None:
        name = 'Save Cache'

    if segment_download_timeout_mins is not None:
        merged_env = dict(env or {})
        merged_env['SEGMENT_DOWNLOAD_TIMEOUT_MINS'] = str(segment_download_timeout_mins)
        env = merged_env

    return action(
        name,
        'actions/cache/save',
        ref=version,
        with_opts=options or None,
        args=args,
        entrypoint=entrypoint,
        condition=condition,
        working_directory=working_directory,
        shell=shell,
        id=id,
        env=env,
        continue_on_error=continue_on_error,
        timeout_minutes=timeout_minutes,
    )


def cache_restore(
    *,
    key: StringLike,
    name: Ostrlike = None,
    version: str = 'v5',
    path: list[StringLike] | None = None,
    restore_keys: list[StringLike] | None = None,
    enable_cross_os_archive: Oboollike = None,
    fail_on_cache_miss: Oboollike = None,
    lookup_only: Oboollike = None,
    segment_download_timeout_mins: Ointlike = None,
    args: Ostrlike = None,
    entrypoint: Ostrlike = None,
    condition: Oboolstr = None,
    working_directory: Ostrlike = None,
    shell: Ostr = None,
    id: Ostr = None,  # noqa: A002
    env: Mapping[str, StringLike] | None = None,
    continue_on_error: Oboollike = None,
    timeout_minutes: Ointlike = None,
) -> Step:
    options: dict[str, object] = {
        'key': key,
        'path': list(path) if path is not None else None,
        'restore-keys': list(restore_keys) if restore_keys is not None else None,
        'ensembleCrossOsArchive': enable_cross_os_archive,
        'fail-on-cache-miss': fail_on_cache_miss,
        'lookup-only': lookup_only,
    }
    options = {key: value for key, value in options.items() if value is not None}

    if name is None:
        name = 'Restore Cache'

    if segment_download_timeout_mins is not None:
        merged_env = dict(env or {})
        merged_env['SEGMENT_DOWNLOAD_TIMEOUT_MINS'] = str(segment_download_timeout_mins)
        env = merged_env

    return action(
        name,
        'actions/cache/restore',
        ref=version,
        with_opts=options or None,
        args=args,
        entrypoint=entrypoint,
        condition=condition,
        working_directory=working_directory,
        shell=shell,
        id=id,
        env=env,
        continue_on_error=continue_on_error,
        timeout_minutes=timeout_minutes,
    )



from __future__ import annotations

import argparse
import os
import subprocess
import sys
from pathlib import Path


DEFAULT_CANDIDATES = ('.yamloom.py', 'yamloom.py')
ENV_VAR = 'YAMLOOM_FILE'


def resolve_target(explicit: str | None) -> Path:
    if explicit:
        return Path(explicit)

    env_value = os.getenv(ENV_VAR)
    if env_value:
        return Path(env_value)

    for candidate in DEFAULT_CANDIDATES:
        path = Path(candidate)
        if path.exists():
            return path

    candidates = ', '.join(DEFAULT_CANDIDATES)
    raise FileNotFoundError(
        f'Could not find workflow generator. Tried: {candidates}. '
        f'Set --file or {ENV_VAR} to override.'
    )


def main() -> int:
    parser = argparse.ArgumentParser(description='Run yamloom workflow generator.')
    parser.add_argument(
        '--file',
        dest='file',
        help='Path to workflow generator script (overrides defaults).',
    )
    args = parser.parse_args()

    try:
        target = resolve_target(args.file)
    except FileNotFoundError as exc:
        print(str(exc), file=sys.stderr)
        return 2

    if not target.exists():
        print(f'Workflow generator not found: {target}', file=sys.stderr)
        return 2

    result = subprocess.run([sys.executable, str(target)])
    return result.returncode


if __name__ == '__main__':
    raise SystemExit(main())



from __future__ import annotations
from yamloom.actions.utils import validate_choice

from typing import TYPE_CHECKING

from ..._yamloom import Step
from ..._yamloom import action
from ..types import (
    Oboollike,
    Oboolstr,
    Ointlike,
    Ostr,
    Ostrlike,
    StringLike,
)

if TYPE_CHECKING:
    from collections.abc import Mapping

__all__ = ['install_rust_tool', 'setup_rust']


def setup_rust(
    *,
    name: Ostrlike = None,
    version: str = 'v1',
    toolchain: Ostrlike = None,
    target: Ostrlike = None,
    components: list[StringLike] | None = None,
    cache: Oboollike = None,
    cache_directories: list[StringLike] | None = None,
    cache_workspaces: list[StringLike] | None = None,
    cache_on_failure: Oboollike = None,
    cache_key: Ostrlike = None,
    cache_shared_key: Ostrlike = None,
    cache_bin: Oboollike = None,
    cache_provider: Ostrlike = None,
    cache_all_crates: Oboollike = None,
    cache_workspace_crates: Oboollike = None,
    matcher: Oboollike = None,
    rustflags: Ostrlike = None,
    override: Oboollike = None,
    rust_src_dir: Ostrlike = None,
    args: Ostrlike = None,
    entrypoint: Ostrlike = None,
    condition: Oboolstr = None,
    working_directory: Ostrlike = None,
    shell: Ostr = None,
    id: Ostr = None,  # noqa: A002
    env: Mapping[str, StringLike] | None = None,
    continue_on_error: Oboollike = None,
    timeout_minutes: Ointlike = None,
) -> Step:
    options: dict[str, object] = {
        'toolchain': toolchain,
        'target': target,
        'components': ','.join(str(s) for s in components)
        if components is not None
        else None,
        'cache': cache,
        'cache-directories': '\n'.join(str(s) for s in cache_directories)
        if cache_directories is not None
        else None,
        'cache-workspaces': '\n'.join(str(s) for s in cache_workspaces)
        if cache_workspaces is not None
        else None,
        'cache-on-failure': cache_on_failure,
        'cache-key': cache_key,
        'cache-shared-key': cache_shared_key,
        'cache-bin': cache_bin,
        'cache-provider': validate_choice(
            'cache_provider', cache_provider, ['github', 'buildjet', 'warpbuild']
        ),
        'cache-all-crates': cache_all_crates,
        'cache-workspace-crates': cache_workspace_crates,
        'matcher': matcher,
        'rustflags': rustflags,
        'override': override,
        'rust-src-dir': rust_src_dir,
    }

    options = {key: value for key, value in options.items() if value is not None}

    if name is None:
        name = 'Setup Rust'

    return action(
        name,
        'actions-rust-lang/setup-rust-toolchain',
        ref=version,
        with_opts=options or None,
        args=args,
        entrypoint=entrypoint,
        condition=condition,
        working_directory=working_directory,
        shell=shell,
        id=id,
        env=env,
        continue_on_error=continue_on_error,
        timeout_minutes=timeout_minutes,
    )


def install_rust_tool(
    *,
    tool: list[StringLike],
    name: Ostrlike = None,
    version: str = 'v1',
    checksum: Oboollike = None,
    fallback: Ostrlike = None,
    args: Ostrlike = None,
    entrypoint: Ostrlike = None,
    condition: Oboolstr = None,
    working_directory: Ostrlike = None,
    shell: Ostr = None,
    id: Ostr = None,  # noqa: A002
    env: Mapping[str, StringLike] | None = None,
    continue_on_error: Oboollike = None,
    timeout_minutes: Ointlike = None,
) -> Step:
    if not tool:
        msg = "at least one 'tool' must be specified"
        raise ValueError(msg)

    options: dict[str, object] = {
        'tool': ','.join(str(s) for s in tool),
        'checksum': checksum,
        'fallback': validate_choice(
            'fallback', fallback, ['none', 'cargo-binstall', 'cargo-install']
        ),
    }

    options = {key: value for key, value in options.items() if value is not None}

    if name is None:
        suffix = 's' if len(tool) > 1 else ''
        name = f'Install Rust Tool{suffix}'

    return action(
        name,
        'taiki-e/install-action',
        ref=version,
        with_opts=options or None,
        args=args,
        entrypoint=entrypoint,
        condition=condition,
        working_directory=working_directory,
        shell=shell,
        id=id,
        env=env,
        continue_on_error=continue_on_error,
        timeout_minutes=timeout_minutes,
    )
